<?php
	/*
        Dispatch les déclinaisons des produits en plusieurs produits en ne gardant
        que certains attributs jumulés
		
		Usage :
		split_products <attributes>
                    
        Avec :
        - attributes : liste des numéros des attributs à garder ensembles, séparée par des virgules
	*/	
    
    $opts = getopt("", array("cfg:", "attributes:", "photo"));

	if (!isset($opts["attributes"]) || empty($opts["attributes"]) || !isset($opts["cfg"]) || empty($opts["cfg"]))
	{
        echo "Usage php ".$argv[0]." --cfg /path/to/config.inc.php directory --attributes <x,y>\n";
        die();
    }
    
	// Autodétection de l'emplacement de Prestashop (attention au lien symbolique sur plateforme de dev)    
	    
	// Inclusion de la configuration
    require($opts["cfg"].'/config.inc.php');		
    
	class ProductSpliter
	{
        private $db;
        private $systemLang;

		public function __construct()
		{
            $this->systemLang = Configuration::get('PS_LANG_DEFAULT');
		}
		
		/*
			Enregistre une erreur sur le journal de Prestashop
			et également sur la console
		*/
		function logError($msg, $level = 3, $objClass = null, $objId = 0)
		{
			PrestaShopLogger::addLog("Spliter : ".$msg, $level, 1, "Spliter", $objId, true);
			echo $msg."\n";
        }
        
        /*
			Déplace une combinaison dans un autre produit
		*/
		function moveProductAttributeToAnotherProduct($combination, $product)
		{
			// Création nouvelle combinaison
            $newCombination = new Combination();
            
            foreach (Combination::$definition["fields"] as $field => $def)
                $newCombination->$field = $combination->$field;

			$newCombination->id_product = $product->id;
            $newCombination->save();
            
            // Copie des attributs
			$attrs = array();
			foreach ($combination->getAttributesName($this->systemLang) as $attr)
				$attrs[] = $attr["id_attribute"];									
            $newCombination->setAttributes($attrs);
            
            // Recopions les références fournisseurs
            foreach (ProductSupplier::getSupplierCollection($combination->id_product, false) as $ps)
            {
                if ($ps->id_product_attribute == $combination->id)
                {
                    $nps = new ProductSupplier();
                    foreach (ProductSupplier::$definition["fields"] as $field => $def)
                        $nps->$field = $ps->$field;

                    $nps->id_product = $newCombination->id_product;
                    $nps->id_product_attribute = $newCombination->id;

                    $nps->save();
                }
            }
            
            return $newCombination->id;
        }

        /*
         * Supprime du produit les photos qui ne sont pas utilisées par au moins une catégorie
         */
        function menagePhoto($product)
        {
            $id_product = $product->id;
            foreach ($this->db->ExecuteS("SELECT id_image FROM "._DB_PREFIX_."image WHERE id_product = ".$id_product." AND id_image NOT IN (SELECT id_image FROM "._DB_PREFIX_."product_attribute_image WHERE id_product_attribute IN (SELECT id_product_attribute FROM "._DB_PREFIX_."product_attribute WHERE id_product = ".$id_product.")) ORDER BY id_image") as $row)
            {
                $img = new Image($row["id_image"]);
                $img->delete();                
            }
            Image::deleteCover($id_product);

            foreach ($product->getImages($this->systemLang) as $row)
            {
                $product->setCoverWs($row["id_image"]);
                break;
            }
        }
        
		/*
			Importation
		*/
		function run($attributes)
		{
            // Base de données
            $this->db = Db::getInstance();
            if (!$this->db)
                die ("Erreur d'accès à la base de données");

            // Pas de cache !
            $this->db->disableCache();

            // Chargement de tous les produits
            $products = $this->db->ExecuteS("SELECT id_product FROM "._DB_PREFIX_."product ORDER BY id_product");
            $c_products = count($products);
            $c_idx = 0;
            foreach($products as $row)
            {                
                echo sprintf("%d/%d\n", ++$c_idx, $c_products);

                // Chargement du produit ...
                $product = new Product($row["id_product"]);

                // Calcul du dispatch ...
                $dispatch = array();
                foreach (Product::getProductAttributesIds($product->id) as $pa)
                {
                    $id_pa = $pa["id_product_attribute"];
                                            
                    $k = "r";
                    foreach ($this->db->ExecuteS(
                                sprintf("SELECT id_attribute_group, id_attribute FROM "._DB_PREFIX_."attribute WHERE id_attribute_group NOT IN (%s) AND id_attribute IN (SELECT id_attribute FROM "._DB_PREFIX_."product_attribute_combination WHERE id_product_attribute = %d) ORDER BY id_attribute_group", 
                                    join(',', $attributes), $id_pa)) as $row)
                        $k .= "-".$row["id_attribute"];

                    if (!isset($dispatch[$k]))
                        $dispatch[$k] = array();
                    
                    $dispatch[$k][] = $id_pa;
                }
                
                if (count($dispatch) > 1)
                {
                    // Il faut dispatcher le produit !
                    $newProductsIds = [];
                    $newProductsIds[] = $product->id;
                                        
                    $idx = 0;
                    foreach ($dispatch as $attrs)
                    {
                        if ($idx > 0)
                        {
                            // on ignore le premier, on créer les autres ...
                            $new_product = new Product();
                            foreach (Product::$definition["fields"] as $field => $def)
                                $new_product->$field = $product->$field;

                            $new_product->save();
                            $newProductsIds[] = $new_product->id;
                            
                            // Attribution des catégories
                            Category::duplicateProductCategories($product->id, $new_product->id);

                            // Copie des tags
                            //duplicateSpecificPrices
                            Product::duplicateTags($product->id, $new_product->id);
                            Product::duplicateDownload($product->id, $new_product->id);
                            Product::duplicateAttachments($product->id, $new_product->id);
                            Product::duplicateFeatures($product->id, $new_product->id);
                            Product::duplicateCustomizationFields($product->id, $new_product->id);
                            Product::duplicateAccessories($product->id, $new_product->id);
                            
                            // Déplacement attributs ...
                            $combination_images = array();

                            $combinations_to_delete = [];

                            $doPhotoClean = false;
                            foreach ($attrs as $attr)
                            {
                                $pa_id = $attr;
                                $cmb = new Combination($pa_id);
                                $npa_id = $this->moveProductAttributeToAnotherProduct($cmb, $new_product);

                                if ($result_images = Product::_getAttributeImageAssociations($pa_id)) 
                                {
                                    $combination_images['old'][$pa_id] = $result_images;
                                    $combination_images['new'][$npa_id] = $result_images;

                                    if (count($result_images) > 0)
                                        $doPhotoClean = true;
                                }                                

                                $combinations_to_delete[] = $cmb;
                            }

                            // Bougeons les photos ?            
                            Image::duplicateProductImages($product->id, $new_product->id, $combination_images);
                            
                            // On fait le ménage dans les photos uniquement si les photos sont associées aux déclinaisons
                            if ($doPhotoClean)
                                $this->menagePhoto($new_product);

                            // Suppression anciennes combinaisons
                            foreach ($combinations_to_delete as $c)
                                $c->delete();
                        }
                        $idx++;
                    }
                                        
                    $this->logError(sprintf("éclatement produit %d en %s", $product->id, join(", ", $newProductsIds)), 1);
                }
            }

            // Alignement date de produits
            //$this->db->execute("UPDATE ps_product_shop AS o SET date_add = (SELECT MIN(p.date_add) FROM ps_product AS p  LEFT JOIN ps_polarislink_products_group AS g ON (p.id_product = g.id_product) WHERE id_group = (SELECT id_group FROM ps_polarislink_products_group WHERE id_product = o.id_product) )");                
            //$this->db->execute("UPDATE ps_product p SET date_add = (SELECT date_add FROM ps_product_shop WHERE id_product = p.id_product)");
        }
    }
    
	$spliter = new ProductSpliter();
    $spliter->run(
                explode(",", $opts["attributes"])
            );
